import { CustomSuit } from '../Products/CustomSuit';

/**
 * SuitBuilder - класс, предназначенный для пошагового создания экземпляров костюмов.
 *
 * Этот класс использует паттерн "Строитель" для добавления различных компонентов
 * (пиджак, брюки, жилет) в кастомный костюм.
 */
export class SuitBuilder {
    private customSuit: CustomSuit;

    /**
     * Конструктор класса SuitBuilder.
     *
     * Инициализирует новый экземпляр кастомного костюма, который будет построен.
     */
    constructor() {
        this.customSuit = new CustomSuit();
    }

    /**
     * Метод для добавления пиджака в костюм.
     *
     * Этот метод добавляет компонент "Пиджак" в кастомный костюм.
     */
    createJacket(): void {
        this.customSuit.addComponent("Пиджак");
    }

    /**
     * Метод для добавления брюк в костюм.
     *
     * Этот метод добавляет компонент "Брюки" в кастомный костюм.
     */
    createTrousers(): void {
        this.customSuit.addComponent("Брюки");
    }

    /**
     * Метод для добавления жилета в костюм.
     *
     * Этот метод добавляет компонент "Жилет" в кастомный костюм.
     */
    createVest(): void {
        this.customSuit.addComponent("Жилет");
    }

    /**
     * Метод для получения готового кастомного костюма.
     *
     * @returns {CustomSuit} Возвращает созданный экземпляр кастомного костюма.
     */
    getResult(): CustomSuit {
        return this.customSuit;
    }

    /**
     * Метод для сброса текущего костюма и начала сборки нового.
     *
     * Этот метод инициализирует новый экземпляр кастомного костюма,
     * позволяя строителю начинать новую сборку с чистого листа.
     */
    reset(): void {
        this.customSuit = new CustomSuit();
    }
}
