import { ComplexClothingItem } from './Contracts/ComplexClothingItem';

/**
 * Класс, представляющий мужской костюм.
 * Реализует интерфейс ComplexClothingItem.
 */
export class Suit implements ComplexClothingItem {
    // Массив для хранения компонентов костюма.
    private components: string[] = [];

    // Объект для хранения цен компонентов.
    private componentPrices: { [key: string]: number } = {
        "Пиджак": 10000,
        "Брюки": 5000,
        "Жилет": 4000,
    };

    /**
     * Метод для получения описания мужского костюма.
     * @returns {string} - Описание мужского костюма.
     */
    getDescription(): string {
        // Возвращает описание костюма.
        return "Мужской костюм";
    }

    /**
     * Метод для получения цены мужского костюма.
     * @returns {number} - Цена мужского костюма.
     */
    getPrice(): number {
        // Вычисляет общую цену на основе добавленных компонентов.
        return this.components.reduce((total, component) => {
            // Суммирует цены компонентов.
            return total + (this.componentPrices[component] || 0);
        }, 0);
    }

    /**
     * Метод для добавления компонента к мужскому костюму.
     * @param {string} component - Название компонента.
     */
    addComponent(component: string): void {
        // Добавляет компонент в массив.
        this.components.push(component);
    }

    /**
     * Метод для получения компонентов, из которых состоит костюм.
     * @returns {string[]} - Массив названий компонентов.
     */
    getComponents(): string[] {
        // Возвращает массив компонентов.
        return this.components;
    }
}
