import { ClothingPrototype } from './ClothingPrototype';

/**
 * Реестр прототипов, позволяющий регистрировать и получать прототипы предметов одежды.
 */
export class PrototypeRegistry {
    // Хранит зарегистрированные прототипы по ключу.
    private prototypes: { [key: string]: ClothingPrototype } = {};

    /**
     * Метод для регистрации прототипа по ключу.
     * @param key - Ключ для идентификации прототипа.
     * @param prototype - Прототип предмета одежды.
     */
    register(key: string, prototype: ClothingPrototype): void {
        this.prototypes[key] = prototype; // Сохраняет прототип в реестре.
    }

    /**
     * Метод для получения прототипа по ключу.
     * @param key - Ключ для идентификации прототипа.
     * @returns {ClothingPrototype} - Прототип предмета одежды.
     */
    get(key: string): ClothingPrototype {
        return this.prototypes[key]; // Возвращает прототип из реестра.
    }
}
